/**
\file avr_gpio.h
\brief Library for using GPIO as Digital I/O 

\verbatim

Created: 08-08-2020 06:12:10 PM
Author : Arnab Kumar Das
Website: www.ArnabKumarDas.com
Microcontroller Supported: ATmega48A/PA/88A/PA/168A/PA/328/P or Arduino UNO/NANO/MINI

Copyright (c) 2014-2020 Arnab Kumar Das
You may freely modify and share this code, as long as you keep this
notice intact (including the links above).
Licensed under the Creative Commons BY-SA 3.0 license:

  http://creativecommons.org/licenses/by-sa/3.0/

Disclaimer: To the extent permitted by law, Arnab Kumar Das provides this work
without any warranty.  It might be defective, in which case you agree
to be responsible for all resulting costs and damages.

               +-\/-+
         PC6  1|    |28  PC5 (A5/ADC5)
RXD (D0) PD0  2|    |27  PC4 (A4/ADC4)
TXD (D1) PD1  3|    |26  PC3 (A3/ADC3)
    (D2) PD2  4|    |25  PC2 (A2/ADC2)
PWM (D3) PD3  5|    |24  PC1 (A1/ADC1)
XCK (D4) PD4  6|    |23  PC0 (A0/ADC0)
         VCC  7|    |22  GND
         GND  8|    |21  AREF
         PB6  9|    |20  AVCC
         PB7 10|    |19  PB5 (D13)
PWM (D5) PD5 11|    |18  PB4 (D12)
PWM (D6) PD6 12|    |17  PB3 (D11) PWM
    (D7) PD7 13|    |16  PB2 (D10) PWM
    (D8) PB0 14|    |15  PB1 (D9)  PWM
               +----+

\endverbatim

*/

#ifndef AVR_GPIO_H_
#define AVR_GPIO_H_

#include <avr/io.h>

#pragma message ( "avr_gpio.h included" )

/** @name GPIO_MODE
 *  Options for GPIO Operation Mode
 */
/**@{*/
#define GPIO_MODE_OUTPUT 				0x00 /**< Sets the GPIO Mode of Operation to : Output */
#define GPIO_MODE_TRI_STATE             0x02 /**< Sets the GPIO Mode of Operation to : Tri-state (Hi-Z) */
#define GPIO_MODE_INPUT_WITH_PULL_UP    0x01 /**< Sets the GPIO Mode of Operation to : Input with Internal Pull Up */
#define GPIO_MODE_INPUT_WITHOUT_PULL_UP 0x02 /**< Sets the GPIO Mode of Operation to : Input without Internal Pull Up */
/**@{*/

/** @name GPIO_PIN_STATE
 *  Options for Digital I/O Pin State
 */
/**@{*/
#define LOW	   0x00 /**< Sets the GPIO Pin State to : Output LOW / 0V / GND */
#define HIGH   0x01 /**< Sets the GPIO Pin State to : Output HIGH / ~5V / VCC */
#define TOGGLE 0x02 /**< Sets the GPIO Pin State to : Toggle the Output Level */
/**@{*/

/** @name GPIO_PORT_DATA_REGISTER
 *  Options for GPIO Port Data Register
 */
/**@{*/
#define PB &PORTB
#define PC &PORTC
#define PD &PORTD
/**@{*/

/** @name GPIO_PORT_BIT_MASK
 *  Options for GPIO Port Bit Mask
 */
/**@{*/
#define P0  (1<<0)
#define P1  (1<<1)
#define P2  (1<<2)
#define P3  (1<<3)
#define P4  (1<<4)
#define P5  (1<<5)
#define P6  (1<<6)
#define P7  (1<<7)
#define ALL_PINS 0xFF
/**@{*/

/** @name GPIO_ARDUINO_PINS
 *  Options for GPIO Digital I/O Pin in Arduino Style
 */
/**@{*/
#define D0  PD,P0
#define D1  PD,P1
#define D2  PD,P2
#define D3  PD,P3
#define D4  PD,P4
#define D5  PD,P5
#define D6  PD,P6
#define D7  PD,P7
#define D8  PB,P0
#define D9  PB,P1
#define D10 PB,P2
#define D11 PB,P3
#define D12 PB,P4
#define D13 PB,P5
#define D14 PC,P0
#define D15 PC,P1
#define D16 PC,P2
#define D17 PC,P3
#define D18 PC,P4
#define D19 PC,P5
#define A0  PC,P0
#define A1  PC,P1
#define A2  PC,P2
#define A3  PC,P3
#define A4  PC,P4
#define A5  PC,P5
/**@{*/

/** @name GPIO_ATMEGA_PINS
 *  Options for GPIO Digital I/O Pin in Atmel AVR Style
 */
/**@{*/
#ifdef  PD0
#undef  PD0
#define PD0 PD,P0
#endif
#ifdef  PD1
#undef  PD1
#define PD1 PD,P1
#endif
#ifdef  PD2
#undef  PD2
#define PD2 PD,P2
#endif
#ifdef  PD3
#undef  PD3
#define PD3 PD,P3
#endif
#ifdef  PD4
#undef  PD4
#define PD4 PD,P4
#endif
#ifdef  PD5
#undef  PD5
#define PD5 PD,P5
#endif
#ifdef  PD6
#undef  PD6
#define PD6 PD,P6
#endif
#ifdef  PD7
#undef  PD7
#define PD7 PD,P7
#endif
#ifdef  PB0
#undef  PB0
#define PB0 PB,P0
#endif
#ifdef  PB1
#undef  PB1
#define PB1 PB,P1
#endif
#ifdef  PB2
#undef  PB2
#define PB2 PB,P2
#endif
#ifdef  PB3
#undef  PB3
#define PB3 PB,P3
#endif
#ifdef  PB4
#undef  PB4
#define PB4 PB,P4
#endif
#ifdef  PB5
#undef  PB5
#define PB5 PB,P5
#endif
#ifdef  PC0
#undef  PC0
#define PC0 PC,P0
#endif
#ifdef  PC1
#undef  PC1
#define PC1 PC,P1
#endif
#ifdef  PC2
#undef  PC2
#define PC2 PC,P2
#endif
#ifdef  PC3
#undef  PC3
#define PC3 PC,P3
#endif
#ifdef  PC4
#undef  PC4
#define PC4 PC,P4
#endif
#ifdef  PC5
#undef  PC5
#define PC5 PC,P5
#endif
/**@{*/

/** @name GPIO_GLOBAL_PULL_UP_CONTROL
 *  Public Functions to Control Global GPIO Pull Up
 */
/**@{*/
inline void GPIO_DisableGlobalPullUp();
inline void GPIO_EnableGlobalPullUp();
/**@{*/

/** @name GPIO_CONFIGURATION
 *  Public Functions to Configure GPIO as Digital I/O
 */
/**@{*/
inline void GPIO_Init(volatile uint8_t* Port,uint8_t Pin,uint8_t PinMode);
inline void GPIO_InitPin(uint8_t Pin,uint8_t PinMode);
inline void GPIO_DeInit(volatile uint8_t* Port,uint8_t Pin);
inline void GPIO_DeInitPin(uint8_t Pin);
/**@{*/

/** @name GPIO_PIN_READ_WRITE
 *  Public Functions to Read and Write Digital I/O Pin
 */
/**@{*/
inline uint8_t GPIO_Read(volatile uint8_t* Port, uint8_t Pin);
inline uint8_t GPIO_ReadPin(uint8_t Pin);
inline void GPIO_Write(volatile uint8_t* Port, uint8_t Pin, uint8_t PinState);
inline void GPIO_Toggle(volatile uint8_t* Port, uint8_t Pin);
inline void GPIO_WritePinLow(uint8_t Pin);
inline void GPIO_WritePinHigh(uint8_t Pin);
inline void GPIO_WritePinToggle(uint8_t Pin);
/**@{*/

/**
 *  \brief Public Function to Disable Global Pull Up
 *  
 *  \return void
 */
inline void GPIO_DisableGlobalPullUp()
{
	MCUCR |= 1<<PUD;
}

/**
 *  \brief Public Function to Enable Global Pull Up
 *  
 *  \return void
 */
inline void GPIO_EnableGlobalPullUp()
{
	MCUCR &= ~(1<<PUD);
}

/**
 *  \brief Public Function to Configure and Initialize Digital I/O Pin
 *  
 *  \param [in] Port : Memory Address of Data Register PORTx of I/O Port
 *  \param [in] Pin : Bit Mask of Data Register PORTx 
 *  \param [in] PinMode : Digital I/O Pin Operation Mode
 *  \return void
 */
inline void GPIO_Init(volatile uint8_t* Port, uint8_t Pin, uint8_t PinMode)
{
	if (PinMode == GPIO_MODE_OUTPUT)
	{
		*(Port-0x01) |= Pin;
	}
	else if(PinMode == GPIO_MODE_INPUT_WITH_PULL_UP)
	{
		*(Port-0x01) &= ~(Pin);
		*Port |= Pin;
		GPIO_EnableGlobalPullUp();
	}
	else if(PinMode == GPIO_MODE_INPUT_WITHOUT_PULL_UP)
	{
		*(Port-0x01) &= ~(Pin);
		*Port |= ~(Pin);
	}
}

/**
 *  \brief Public Function to Configure and Initialize Digital I/O Pin
 *
 *  \param [in] Pin : Digital I/O Pin Number 0 ... 19
 *  \param [in] PinMode : Digital I/O Pin Operation Mode
 *  \return void
 */
inline void GPIO_InitPin(uint8_t Pin, uint8_t PinMode)
{
	if (Pin < 8)			/* pin 0 = PD0, ..., 7 = PD7 */
	{
		GPIO_Init(&PORTD,(1<<Pin),PinMode);
	}
	else if (Pin < 14)		/* pin 8 = PB0, ..., 13 = PB5 (PB6 and PB7 reserved for External Clock) */
	{
		GPIO_Init(&PORTB,(1<<(Pin - 8)),PinMode);
	}
	else if (Pin < 21)		/* pin 14 = PC0, ..., 19 = PC5 (PC6 is Reset, PC7 doesn't exist) */
	{
		GPIO_Init(&PORTC,(1<<(Pin - 14)),PinMode);
	}
}

/**
 *  \brief Public Function to De-Initialize Digital I/O Pin
 *  
 *  \param [in] Port : Memory Address of Data Register PORTx of I/O Port
 *  \param [in] Pin : Bit Mask of Data Register PORTx
 *  \return void
 */
inline void GPIO_DeInit(volatile uint8_t* Port, uint8_t Pin)
{
	*(Port-0x01) &= ~(Pin);
	*(Port-0x02) &= ~(Pin);
	*(Port) &= ~(Pin);
}


/**
 *  \brief Public Function to De-Initialize Digital I/O Pin
 *
 *  \param [in] Pin : Digital I/O Pin Number 0 ... 19
 *  \return void
 */
inline void GPIO_DeInitPin(uint8_t Pin)
{
	if (Pin < 8)			/* pin 0 = PD0, ..., 7 = PD7 */
	{
		GPIO_DeInit(&PORTD,(1<<Pin));
	}
	else if (Pin < 14)		/* pin 8 = PB0, ..., 13 = PB5 (PB6 and PB7 reserved for External Clock) */
	{
		GPIO_DeInit(&PORTB,(1<<(Pin - 8)));
	}
	else if (Pin < 21)		/* pin 14 = PC0, ..., 19 = PC5 (PC6 is Reset, PC7 doesn't exist) */
	{
		GPIO_DeInit(&PORTC,(1<<(Pin - 14)));
	}
}

/**
 *  \brief Public Function to Read Digital I/O Pin
 *  
 *  \param [in] Port : Memory Address of Data Register PORTx of I/O Port
 *  \param [in] Pin : Bit Mask of Data Register PORTx
 *  \return  Digital I/O Pin State 
 */
inline uint8_t GPIO_Read(volatile uint8_t* Port, uint8_t Pin)
{
	return *(Port - 0x02) & (Pin);
}

/**
 *  \brief Public Function to Write Output Value Digital I/O Pin
 *  
 *  \param [in] Port : Memory Address of Data Register PORTx of I/O Port
 *  \param [in] Pin : Bit Mask of Data Register PORTx
 *  \param [in] PinState : Digital I/O Pin State
 *  \return void
 */
inline void GPIO_Write(volatile uint8_t* Port, uint8_t Pin, uint8_t PinState)
{
	if (PinState == HIGH)
	{
		*Port |= Pin;
	}
	else if (PinState == LOW)
	{
		*Port &= ~(Pin);
	}
	else if (PinState == TOGGLE)
	{
		*(Port-0x02) = Pin;
	}
}

/**
 *  \brief Public Function to Toggle Output Value of Digital I/O Pin
 *  
 *  \param [in] Port : Memory Address of Data Register PORTx of I/O Port
 *  \param [in] Pin : Bit Mask of Data Register PORTx
 *  \return void
 */
inline void GPIO_Toggle(volatile uint8_t* Port, uint8_t Pin)
{
	*(Port-0x02) |= Pin; /* Uses a Single Instruction to Toggle a Bit */
	
	//*Port ^= Pin; /* Uses 4 Instructions to Toggle a Bit */
}

/**
 *  \brief Public Function to Read Digital I/O Pin
 *  
 *  \param [in] Pin : Digital I/O Pin Number 0 ... 19
 *  \return  Digital I/O Pin State
 */
inline uint8_t GPIO_ReadPin(uint8_t Pin)
{
	if (Pin < 8)			/* pin 0 = PD0, ..., 7 = PD7 */
	{
		return GPIO_Read(&PORTD,(1<<Pin));
	}
	else if (Pin < 14)		/* pin 8 = PB0, ..., 13 = PB5 (PB6 and PB7 reserved for External Clock) */
	{
		return GPIO_Read(&PORTB,(1<<(Pin - 8)));
	}
	else if (Pin < 21)		/* pin 14 = PC0, ..., 19 = PC5 (PC6 is Reset, PC7 doesn't exist) */
	{
		return GPIO_Read(&PORTC,(1<<(Pin - 14)));
	}
}

/**
 *  \brief Public Function to Write Output Value of Digital I/O Pin to HIGH
 *  
 *  \param [in] Pin : Digital I/O Pin Number 0 ... 19
 *  \return void
 */
inline void GPIO_WritePinHigh(uint8_t Pin)
{
	if (Pin < 8)			/* pin 0 = PD0, ..., 7 = PD7 */
	{
		GPIO_Write(&PORTD,(1<<Pin),HIGH);
	}
	else if (Pin < 14)		/* pin 8 = PB0, ..., 13 = PB5 (PB6 and PB7 reserved for External Clock) */
	{
		GPIO_Write(&PORTB,(1<<(Pin - 8)),HIGH);
	}
	else if (Pin < 21)		/* pin 14 = PC0, ..., 19 = PC5 (PC6 is Reset, PC7 doesn't exist) */
	{
		GPIO_Write(&PORTC,(1<<(Pin - 14)),HIGH);
	}
}

/**
 *  \brief Public Function to Write Output Value of Digital I/O Pin to LOW
 *  
 *  \param [in] Pin : Digital I/O Pin Number 0 ... 19
 *  \return void
 */
inline void GPIO_WritePinLow(uint8_t Pin)
{
	if (Pin < 8)			/* pin 0 = PD0, ..., 7 = PD7 */
	{
		GPIO_Write(&PORTD,(1<<Pin),LOW);
	}
	else if (Pin < 14)		/* pin 8 = PB0, ..., 13 = PB5 (PB6 and PB7 reserved for External Clock) */
	{
		GPIO_Write(&PORTB,(1<<(Pin - 8)),LOW);
	}
	else if (Pin < 21)		/* pin 14 = PC0, ..., 19 = PC5 (PC6 is Reset, PC7 doesn't exist) */
	{
		GPIO_Write(&PORTC,(1<<(Pin - 14)),LOW);
	}
}

/**
 *  \brief Public Function to Write Output Value of Digital I/O Pin to TOGGLE
 *  
 *  \param [in] Pin : Digital I/O Pin Number 0 ... 19
 *  \return void
 */
inline void GPIO_WritePinToggle(uint8_t Pin)
{
	if (Pin < 8)			/* pin 0 = PD0, ..., 7 = PD7 */
	{
		GPIO_Write(&PORTD,(1<<Pin),TOGGLE);
	}
	else if (Pin < 14)		/* pin 8 = PB0, ..., 13 = PB5 (PB6 and PB7 reserved for External Clock) */
	{
		GPIO_Write(&PORTB,(1<<(Pin - 8)),TOGGLE);
	}
	else if (Pin < 21)		/* pin 14 = PC0, ..., 19 = PC5 (PC6 is Reset, PC7 doesn't exist) */
	{
		GPIO_Write(&PORTC,(1<<(Pin - 14)),TOGGLE);
	}
}

#endif /* AVR_GPIO_H_ */